var pull = require('pull-stream')
var sort = require('ssb-sort')
var ref = require('ssb-ref')
var h = require('hyperscript')
var self_id = require('../keys').id
var Obv = require('obv')
var ssbThread = require('ssb-thread')

function once (cont) {
  var ended = false
  return function (abort, cb) {
    if(abort) return cb(abort)
    else if (ended) return cb(ended)
    else
      cont(function (err, data) {
        if(err) return cb(ended = err)
        ended = true
        cb(null, data)
      })
  }
}

exports.needs = {
  build_scroller: 'first',
  message_render: 'first',
  message_name: 'first',
  message_compose: 'first',
  message_unbox: 'first',
  sbot_get: 'first',
  sbot_links: 'first'
}

exports.gives = 'screen_view'


exports.create = function (api) {

  function unbox (msg) {
    return 'string' === typeof msg.value.content ? api.message_unbox(msg) : msg
  }

  var getThread = ssbThread.bind(null, {
    get: function (id, cb) { api.sbot_get(id, cb) },
    links: function (opts) { return api.sbot_links(opts) }
  }, unbox)

  return function (id) {
    if(ref.isMsg(id)) {
      var obv = Obv()
      getThread(id, obv.set)

      var meta = {
        type: 'post',
        root: id,
        branch: id //mutated when thread is loaded.
      }

      var composer = api.message_compose(meta, {shrink: false, placeholder: 'Write a reply'})
      var { container, content } = api.build_scroller({ append: composer })

      api.message_name(id, function (err, name) {
        container.title = name
      })

      var rendered = {}
      obv(function (thread) {
        //RENDER, and INSERT into the correct order...

        var branches = sort.heads(thread)
        meta.branch = branches.length > 1 ? branches : branches[0]
        meta.root = thread[0].value.content.root || thread[0].key
        meta.channel = thread[0].value.content.channel

        if (meta.channel) {
          const channelInput = composer.querySelector('input')
          channelInput.value = `#${meta.channel}`
          channelInput.disabled = true
        }

        var recps = thread[0].value.content.recps

        var priv = thread[0].value['private']
        if(priv) {
          if(recps)
            meta.recps = recps
          else
            meta.recps = [thread[0].value.author, self_id]
        }

        var children = container.children
        thread.forEach(function (data) {
          if(!rendered[data.key]) {
            var el = rendered[data.key] = api.message_render(data)
            content.appendChild(el)
          }
        })
      })

      return container
    }
  }
}












